@@ -5,19 +5,19 @@ from __future__ import annotations
 import logging
 from gettext import gettext as _
 
-from gi.repository import Adw, GLib, Gtk
+from gi.repository import GLib, Gtk
 
-from gsecrets.utils import KeyFileFilter, generate_keyfile
+from gsecrets.utils import (
+    KeyFileFilter,
+    generate_keyfile_async,
+    generate_keyfile_finish,
+)
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/create_database.ui")
-class CreateDatabase(Adw.Bin):
+class CreateDatabase(Gtk.Box):
     """Creates a new Safe when invoked"""
 
-    # TODO Add an accelerator for Escape that
-    # calls on_headerbar_back_button_clicked().
-    # Can be done on GTK 4 with GtkShortcutController.
-
     __gtype_name__ = "CreateDatabase"
 
     stack = Gtk.Template.Child()
@@ -36,6 +36,7 @@ class CreateDatabase(Adw.Bin):
     generate_keyfile_button = Gtk.Template.Child()
 
     open_safe_button = Gtk.Template.Child()
+    back_button = Gtk.Template.Child()
 
     new_password = ""
 
@@ -46,7 +47,6 @@ class CreateDatabase(Adw.Bin):
 
         self.database_manager = dbm
         self.window = window
-        self.back_button = window.create_database_headerbar.back_button
 
         self.back_button.props.sensitive = True
         self.back_button.connect("clicked", self.on_headerbar_back_button_clicked)
@@ -75,8 +75,11 @@ class CreateDatabase(Adw.Bin):
         """Back button: Always goes back to the page in which you select the
         authentication method. In the case we are already in that page
         we kill this page."""
+        self.go_back()
+
+    def go_back(self):
         if self.stack.get_visible_child_name() == "select_auth_method":
-            self.window.view = self.window.View.RECENT_FILES
+            self.window.invoke_initial_screen()
         else:
             self.stack.set_visible_child_name("select_auth_method")
             self.clear_input_fields()
@@ -86,7 +89,7 @@ class CreateDatabase(Adw.Bin):
         try:
             is_saved = dbm.set_credentials_finish(result)
         except GLib.Error as err:
-            logging.error("Could not set credentials: %s", err)
+            logging.error("Could not set credentials: %s", err.message)
             self.failure_page()
         else:
             if is_saved:
@@ -154,10 +157,11 @@ class CreateDatabase(Adw.Bin):
                 self.keyfile_generation_page()
             else:
                 self.database_manager.set_credentials_async(
-                    self.new_password, callback=self._on_set_credentials,
+                    self.new_password,
+                    callback=self._on_set_credentials,
                 )
         else:
-            self.window.send_notification(_("Passwords do not Match"))
+            self.window.send_notification(_("Passwords do not match"))
             self.clear_input_fields()
             self.password_repeat_input1.add_css_class("error")
             self.password_repeat_input2.add_css_class("error")
@@ -194,13 +198,14 @@ class CreateDatabase(Adw.Bin):
             keyfile_path = keyfile.get_path()
             logging.debug("New keyfile location: %s", keyfile_path)
 
-            def callback(gfile, result, keyfile_hash):
+            def callback(gfile, result):
                 password = self.new_password
+                keyfile_path = gfile.get_path()
                 try:
-                    gfile.replace_contents_finish(result)
+                    _res, keyfile_hash = generate_keyfile_finish(result)
                 except GLib.Error as err:
-                    logging.debug("Could not create keyfile: %s", err)
-                    self.window.send_notification(_("Could not Create Keyfile"))
+                    logging.debug("Could not create keyfile: %s", err.message)
+                    self.window.send_notification(_("Could not create keyfile"))
                     self.generate_keyfile_button.set_sensitive(True)
                     self.generate_keyfile_button.set_label(_("Generate"))
                 else:
@@ -208,11 +213,13 @@ class CreateDatabase(Adw.Bin):
                         password = ""
 
                     self.database_manager.set_credentials_async(
-                        password, keyfile_path, keyfile_hash,
+                        password,
+                        keyfile_path,
+                        keyfile_hash,
                         self._on_set_credentials,
                     )
 
-            generate_keyfile(keyfile, callback)
+            generate_keyfile_async(keyfile, callback)
 
     @Gtk.Template.Callback()
     def on_finish_button_clicked(self, _widget: Gtk.Button) -> None:
