@@ -1,7 +1,10 @@
 #!/usr/bin/env python3
 from __future__ import annotations
 
+import os
 import secrets
+import stat
+import typing
 from gettext import gettext as _
 
 from Crypto.Cipher import AES
@@ -9,6 +12,9 @@ from Crypto.Random import get_random_bytes
 
 from gi.repository import Gio, GLib, Gtk
 
+if typing.TYPE_CHECKING:
+    from typing import Tuple
+
 
 def format_time(time: GLib.DateTime | None, hours: bool = True) -> str:
     """Displays a UTC DateTime in the local timezone."""
@@ -26,26 +32,44 @@ def create_random_data(bytes_buffer):
     return secrets.token_bytes(bytes_buffer)
 
 
-def generate_keyfile(gfile: Gio.File, callback: Gio.AsyncReadyCallback) -> None:
+def generate_keyfile_sync(gfile: Gio.File) -> str:
     """The callback returns a GFile as its source object and the keyfile hash as
-    its user_data. The caller has to run
-    source_object.replace_contents_finish()."""
+    its user_data. Returns the hash of the keyfile."""
     key = get_random_bytes(32)
     cipher = AES.new(key, AES.MODE_EAX)
     ciphertext, tag = cipher.encrypt_and_digest(create_random_data(96))  # type: ignore
     contents = cipher.nonce + tag + ciphertext  # type: ignore
-    gbytes = GLib.Bytes.new(contents)
-    keyfile_hash = GLib.compute_checksum_for_bytes(GLib.ChecksumType.SHA1, gbytes)
+    keyfile_hash = GLib.compute_checksum_for_data(GLib.ChecksumType.SHA1, contents)
 
-    gfile.replace_contents_bytes_async(
-        gbytes,
+    flags = Gio.FileCreateFlags.REPLACE_DESTINATION | Gio.FileCreateFlags.PRIVATE
+    gfile.replace_contents(
+        contents,
         None,
         False,
-        Gio.FileCreateFlags.REPLACE_DESTINATION,
+        flags,
         None,
-        callback,
-        keyfile_hash,
     )
+    # Sets the file as read-only
+    os.chmod(gfile.get_path(), stat.S_IREAD)
+
+    return keyfile_hash
+
+
+def generate_keyfile_async(gfile: Gio.File, callback: Gio.AsyncReadyCallback) -> None:
+    def generate_keyfile_task(task, obj, _data, _cancellable):
+        try:
+            keyfile_hash = generate_keyfile_sync(obj)
+        except GLib.Error as err:  # pylint: disable=broad-except
+            task.return_error(err)
+        else:
+            task.return_value(keyfile_hash)
+
+    task = Gio.Task.new(gfile, None, callback)
+    task.run_in_thread(generate_keyfile_task)
+
+
+def generate_keyfile_finish(result: Gio.AsyncResult) -> Tuple[bool, str]:
+    return result.propagate_value()
 
 
 class KeyFileFilter:
